@@ -427,6 +427,6 @@ def inspect_main(argv):
 
 if __name__ == '__main__':
     import logging  # type: ignore
-    logging.basicConfig()
+    logging.basicConfig()  # type: ignore
 
     inspect_main(argv=sys.argv[1:])  # type: ignore
