@@ -10,31 +10,37 @@
 """
 import codecs
 import re
-from typing import TYPE_CHECKING
+import warnings
 
 from docutils.core import Publisher
 from docutils.io import FileInput, NullOutput
+from docutils.parsers.rst import Parser as RSTParser
 from docutils.readers import standalone
 from docutils.statemachine import StringList, string2lines
 from docutils.writers import UnfilteredWriter
-from six import text_type
+from six import text_type, iteritems
 from typing import Any, Union  # NOQA
 
+from sphinx.deprecation import RemovedInSphinx30Warning
+from sphinx.locale import __
 from sphinx.transforms import (
     ApplySourceWorkaround, ExtraTranslatableNodes, CitationReferences,
     DefaultSubstitutions, MoveModuleTargets, HandleCodeBlocks, SortIds,
     AutoNumbering, AutoIndexUpgrader, FilterSystemMessages,
-    UnreferencedFootnotesDetector, SphinxSmartQuotes, ManpageLink
+    UnreferencedFootnotesDetector, SphinxSmartQuotes, DoctreeReadEvent, ManpageLink
 )
 from sphinx.transforms import SphinxTransformer
 from sphinx.transforms.compact_bullet_list import RefOnlyBulletListTransform
 from sphinx.transforms.i18n import (
     PreserveTranslatableMessages, Locale, RemoveTranslatableInline,
 )
+from sphinx.transforms.references import SphinxDomains, SubstitutionDefinitionsRemover
 from sphinx.util import logging
 from sphinx.util.docutils import LoggingReporter
+from sphinx.versioning import UIDTransform
 
-if TYPE_CHECKING:
+if False:
+    # For type annotation
     from typing import Any, Dict, List, Tuple, Union  # NOQA
     from docutils import nodes  # NOQA
     from docutils.io import Input  # NOQA
@@ -92,7 +98,9 @@ class SphinxStandaloneReader(SphinxBaseReader):
                   Locale, CitationReferences, DefaultSubstitutions, MoveModuleTargets,
                   HandleCodeBlocks, AutoNumbering, AutoIndexUpgrader, SortIds,
                   RemoveTranslatableInline, FilterSystemMessages, RefOnlyBulletListTransform,
-                  UnreferencedFootnotesDetector, SphinxSmartQuotes, ManpageLink
+                  UnreferencedFootnotesDetector, SphinxSmartQuotes, ManpageLink,
+                  SphinxDomains, SubstitutionDefinitionsRemover, DoctreeReadEvent,
+                  UIDTransform,
                   ]  # type: List[Transform]
 
     def __init__(self, app, *args, **kwargs):
@@ -110,32 +118,25 @@ class SphinxI18nReader(SphinxBaseReader):
     Because the translated texts are partial and they don't have correct line numbers.
     """
 
-    lineno = None  # type: int
     transforms = [ApplySourceWorkaround, ExtraTranslatableNodes, CitationReferences,
                   DefaultSubstitutions, MoveModuleTargets, HandleCodeBlocks,
                   AutoNumbering, SortIds, RemoveTranslatableInline,
                   FilterSystemMessages, RefOnlyBulletListTransform,
-                  UnreferencedFootnotesDetector, SphinxSmartQuotes, ManpageLink]
+                  UnreferencedFootnotesDetector, SphinxSmartQuotes, ManpageLink,
+                  SubstitutionDefinitionsRemover]
 
     def set_lineno_for_reporter(self, lineno):
         # type: (int) -> None
         """Stores the source line number of original text."""
-        self.lineno = lineno
+        warnings.warn('SphinxI18nReader.set_lineno_for_reporter() is deprecated.',
+                      RemovedInSphinx30Warning)
 
-    def new_document(self):
-        # type: () -> nodes.document
-        """Creates a new document object which having a special reporter object for
-        translation.
-        """
-        document = SphinxBaseReader.new_document(self)
-        reporter = document.reporter
-
-        def get_source_and_line(lineno=None):
-            # type: (int) -> Tuple[unicode, int]
-            return reporter.source, self.lineno
-
-        reporter.get_source_and_line = get_source_and_line
-        return document
+    @property
+    def line(self):
+        # type: () -> int
+        warnings.warn('SphinxI18nReader.line is deprecated.',
+                      RemovedInSphinx30Warning)
+        return 0
 
 
 class SphinxDummyWriter(UnfilteredWriter):
@@ -149,6 +150,7 @@ class SphinxDummyWriter(UnfilteredWriter):
 
 
 def SphinxDummySourceClass(source, *args, **kwargs):
+    # type: (Any, Any, Any) -> Any
     """Bypass source object as is to cheat Publisher."""
     return source
 
@@ -157,7 +159,7 @@ class SphinxBaseFileInput(FileInput):
     """A base class of SphinxFileInput.
 
     It supports to replace unknown Unicode characters to '?'. And it also emits
-    Sphinx events ``source-read`` on reading.
+    Sphinx events :event:`source-read` on reading.
     """
 
     def __init__(self, app, env, *args, **kwds):
@@ -198,7 +200,7 @@ class SphinxBaseFileInput(FileInput):
         if lineend == -1:
             lineend = len(error.object)
         lineno = error.object.count(b'\n', 0, error.start) + 1
-        logger.warning('undecodable source characters, replacing with "?": %r',
+        logger.warning(__('undecodable source characters, replacing with "?": %r'),
                        (error.object[linestart + 1:error.start] + b'>>>' +
                         error.object[error.start:error.end] + b'<<<' +
                         error.object[error.end:lineend]),
@@ -274,14 +276,36 @@ class SphinxRSTFileInput(SphinxBaseFileInput):
             return lineno
 
 
+class FiletypeNotFoundError(Exception):
+    pass
+
+
+def get_filetype(source_suffix, filename):
+    # type: (Dict[unicode, unicode], unicode) -> unicode
+    for suffix, filetype in iteritems(source_suffix):
+        if filename.endswith(suffix):
+            # If default filetype (None), considered as restructuredtext.
+            return filetype or 'restructuredtext'
+    else:
+        raise FiletypeNotFoundError
+
+
 def read_doc(app, env, filename):
     # type: (Sphinx, BuildEnvironment, unicode) -> nodes.document
     """Parse a document and convert to doctree."""
-    input_class = app.registry.get_source_input(filename)
+    filetype = get_filetype(app.config.source_suffix, filename)
+    input_class = app.registry.get_source_input(filetype)
     reader = SphinxStandaloneReader(app)
     source = input_class(app, env, source=None, source_path=filename,
                          encoding=env.config.source_encoding)
-    parser = app.registry.create_source_parser(app, filename)
+    parser = app.registry.create_source_parser(app, filetype)
+    if parser.__class__.__name__ == 'CommonMarkParser' and parser.settings_spec == ():
+        # a workaround for recommonmark
+        #   If recommonmark.AutoStrictify is enabled, the parser invokes reST parser
+        #   internally.  But recommonmark-0.4.0 does not provide settings_spec for reST
+        #   parser.  As a workaround, this copies settings_spec for RSTParser to the
+        #   CommonMarkParser.
+        parser.settings_spec = RSTParser.settings_spec
 
     pub = Publisher(reader=reader,
                     parser=parser,
@@ -289,7 +313,7 @@ def read_doc(app, env, filename):
                     source_class=SphinxDummySourceClass,
                     destination=NullOutput())
     pub.set_components(None, 'restructuredtext', None)
-    pub.process_programmatic_settings(None, env.settings, None)  # type: ignore
+    pub.process_programmatic_settings(None, env.settings, None)
     pub.set_source(source, filename)
     pub.publish()
     return pub.document
